@@ -48,11 +48,20 @@
 #define NONCE_LEN 32
 #define MAX_TEMPLATES 255
 
+#define ADD_NONCE "&nonce="
+#define ADD_OTP "&otp="
+#define ADD_ID "?id="
+
+#define TEMPLATE_FORMAT_OLD 1
+#define TEMPLATE_FORMAT_NEW 2
+
 struct ykclient_st
 {
   const char *ca_path;
+  const char *ca_info;
   size_t num_templates;
   char **url_templates;
+  int template_format;
   char last_url[256];
   unsigned int client_id;
   size_t keylen;
@@ -80,15 +89,13 @@ struct ykclient_handle_st
 };
 
 const char *default_url_templates[] = {
-  "http://api.yubico.com/wsapi/2.0/verify?id=%d&otp=%s",
-  "http://api2.yubico.com/wsapi/2.0/verify?id=%d&otp=%s",
-  "http://api3.yubico.com/wsapi/2.0/verify?id=%d&otp=%s",
-  "http://api4.yubico.com/wsapi/2.0/verify?id=%d&otp=%s",
-  "http://api5.yubico.com/wsapi/2.0/verify?id=%d&otp=%s",
+  "http://api.yubico.com/wsapi/2.0/verify",
+  "http://api2.yubico.com/wsapi/2.0/verify",
+  "http://api3.yubico.com/wsapi/2.0/verify",
+  "http://api4.yubico.com/wsapi/2.0/verify",
+  "http://api5.yubico.com/wsapi/2.0/verify",
 };
 
-const size_t default_num_templates = 5;
-
 /** Initialise the global context for the library
  *
  * This function is not thread safe.  It must be invoked successfully
@@ -99,6 +106,7 @@ const size_t default_num_templates = 5;
 ykclient_rc
 ykclient_global_init (void)
 {
+
   if (curl_global_init (CURL_GLOBAL_ALL) != 0)
     return YKCLIENT_CURL_INIT_ERROR;
   return YKCLIENT_OK;
@@ -141,6 +149,7 @@ ykclient_init (ykclient_t ** ykc)
   memset (p, 0, (sizeof (*p)));
 
   p->ca_path = NULL;
+  p->ca_info = NULL;
 
   p->key = NULL;
   p->keylen = 0;
@@ -168,9 +177,9 @@ ykclient_init (ykclient_t ** ykc)
   /*
    * Set the default URLs (these can be overridden later)
    */
-  ykclient_set_url_templates (p,
-			      sizeof (default_url_templates) /
-			      sizeof (char *), default_url_templates);
+  ykclient_set_url_bases (p,
+			  sizeof (default_url_templates) /
+			  sizeof (char *), default_url_templates);
 
   return YKCLIENT_OK;
 }
@@ -308,6 +317,11 @@ ykclient_handle_init (ykclient_t * ykc, ykclient_handle_t ** ykh)
       if (ykc->ca_path)
 	{
 	  curl_easy_setopt (easy, CURLOPT_CAPATH, ykc->ca_path);
+        }
+
+      if (ykc->ca_info)
+        {
+          curl_easy_setopt (easy, CURLOPT_CAINFO, ykc->ca_info);
 	}
 
       curl_easy_setopt (easy, CURLOPT_WRITEDATA, (void *) data);
@@ -542,6 +556,16 @@ ykclient_set_ca_path (ykclient_t * ykc, const char *ca_path)
   ykc->ca_path = ca_path;
 }
 
+/** Set the CA info, needed for linking with GnuTLS
+ *
+ * Must be called before creating handles.
+ */
+void
+ykclient_set_ca_info (ykclient_t * ykc, const char *ca_info)
+{
+  ykc->ca_info = ca_info;
+}
+
 /** Set a single URL template
  *
  * @param ykc Yubikey client configuration.
@@ -570,6 +594,18 @@ ykclient_set_url_template (ykclient_t * ykc, const char *url_template)
 ykclient_rc
 ykclient_set_url_templates (ykclient_t * ykc, size_t num_templates,
 			    const char **url_templates)
+{
+  ykclient_rc ret = ykclient_set_url_bases(ykc, num_templates, url_templates);
+  if(ret == YKCLIENT_OK)
+    {
+      ykc->template_format = TEMPLATE_FORMAT_OLD;
+    }
+  return ret;
+}
+
+ykclient_rc
+ykclient_set_url_bases (ykclient_t * ykc, size_t num_templates,
+			    const char **url_templates)
 {
   size_t i;
   if (num_templates > MAX_TEMPLATES)
@@ -606,6 +642,8 @@ ykclient_set_url_templates (ykclient_t * ykc, size_t num_templates,
 	  return YKCLIENT_OUT_OF_MEMORY;
 	}
     }
+
+  ykc->template_format = TEMPLATE_FORMAT_NEW;
   return YKCLIENT_OK;
 }
 
@@ -796,6 +834,109 @@ ykclient_generate_nonce (ykclient_t * ykc, char **nonce)
   return YKCLIENT_OK;
 }
 
+static ykclient_rc ykclient_expand_new_url(const char *template,
+    const char *encoded_otp, const char *nonce, int client_id,
+    char **url_exp)
+{
+  size_t len = strlen(template) + strlen(encoded_otp) + strlen(ADD_OTP) + strlen(ADD_ID) + 1;
+  len += snprintf(NULL, 0, "%d", client_id);
+
+  if(nonce)
+    {
+      len += strlen(nonce) + strlen(ADD_NONCE);
+    }
+
+  *url_exp = malloc(len);
+  if(!*url_exp)
+    {
+      return YKCLIENT_OUT_OF_MEMORY;
+    }
+
+  if(nonce)
+    {
+      snprintf(*url_exp, len, "%s" ADD_ID "%d" ADD_NONCE "%s" ADD_OTP "%s",
+	  template, client_id, nonce, encoded_otp);
+    }
+  else
+    {
+      snprintf(*url_exp, len, "%s" ADD_ID "%d" ADD_OTP "%s", template,
+	  client_id, encoded_otp);
+    }
+  return YKCLIENT_OK;
+}
+
+static ykclient_rc ykclient_expand_old_url(const char *template,
+    const char *encoded_otp, const char *nonce, int client_id,
+    char **url_exp)
+{
+  {
+    size_t len;
+    ssize_t wrote;
+
+    len = strlen (template) + strlen (encoded_otp) + 20;
+    *url_exp = malloc (len);
+    if (!*url_exp)
+      {
+        return YKCLIENT_OUT_OF_MEMORY;
+      }
+
+    wrote = snprintf (*url_exp, len, template,
+        client_id, encoded_otp);
+    if (wrote < 0 || (size_t) wrote > len)
+      {
+        return YKCLIENT_FORMAT_ERROR;
+      }
+  }
+
+  if (nonce)
+    {
+      /* Create new URL with nonce in it. */
+      char *nonce_url, *otp_offset;
+      size_t len;
+      ssize_t wrote;
+
+      len =
+        strlen (*url_exp) + strlen (ADD_NONCE) + strlen (nonce) +
+        1;
+      nonce_url = malloc (len + 4);	/* avoid valgrind complaint */
+      if (!nonce_url)
+        {
+          return YKCLIENT_OUT_OF_MEMORY;
+        }
+
+      /* Find the &otp= in url and insert ?nonce= before otp. Must get
+       *  sorted headers since we calculate HMAC on the result.
+       *
+       * XXX this will break if the validation protocol gets a parameter that
+       * sorts in between "nonce" and "otp", because the headers we sign won't
+       * be alphabetically sorted if we insert the nonce between "nz" and "otp".
+       * Also, we assume that everyone will have at least one parameter ("id=")
+       * before "otp" so there is no need to search for "?otp=".
+       */
+      otp_offset = strstr (*url_exp, ADD_OTP);
+      if (otp_offset == NULL)
+        {
+          /* point at \0 at end of url in case there is no otp */
+          otp_offset = *url_exp + len;
+        }
+
+      /* break up ykc->url where we want to insert nonce */
+      *otp_offset = 0;
+
+      wrote = snprintf (nonce_url, len, "%s" ADD_NONCE "%s&%s", *url_exp,
+          nonce, otp_offset + 1);
+      if (wrote < 0 || (size_t) wrote + 1 != len)
+        {
+          free (nonce_url);
+          return YKCLIENT_FORMAT_ERROR;
+        }
+
+      free (*url_exp);
+      *url_exp = nonce_url;
+    }
+    return YKCLIENT_OK;
+}
+
 /** Expand URL templates specified with set_url_templates
  *
  * Expands placeholderss or inserts additional parameters for nonce,
@@ -833,79 +974,22 @@ ykclient_expand_urls (ykclient_t * ykc, ykclient_handle_t * ykh,
 
   for (i = 0; i < ykc->num_templates; i++)
     {
-      {
-	size_t len;
-	ssize_t wrote;
-
-	len = strlen (ykc->url_templates[i]) + strlen (encoded_otp) + 20;
-	ykh->url_exp[i] = malloc (len);
-	if (!ykh->url_exp[i])
-	  {
-	    out = YKCLIENT_OUT_OF_MEMORY;
-	    goto finish;
-	  }
-
-	wrote = snprintf (ykh->url_exp[i], len, ykc->url_templates[i],
-			  ykc->client_id, encoded_otp);
-	if (wrote < 0 || (size_t) wrote > len)
-	  {
-	    out = YKCLIENT_FORMAT_ERROR;
-	    goto finish;
-	  }
-      }
-
-      if (nonce)
+      ykclient_rc ret;
+      if(ykc->template_format == TEMPLATE_FORMAT_OLD)
+        {
+	  ret = ykclient_expand_old_url(ykc->url_templates[i],
+	      encoded_otp, nonce, ykc->client_id, &ykh->url_exp[i]);
+        }
+      else
 	{
-	  /* Create new URL with nonce in it. */
-	  char *nonce_url, *otp_offset;
-	  size_t len;
-	  ssize_t wrote;
-
-#define ADD_NONCE "&nonce="
-	  len =
-	    strlen (ykh->url_exp[i]) + strlen (ADD_NONCE) + strlen (nonce) +
-	    1;
-	  nonce_url = malloc (len + 4);	/* avoid valgrind complaint */
-	  if (!nonce_url)
-	    {
-	      out = YKCLIENT_OUT_OF_MEMORY;
-	      goto finish;
-	    }
-
-	  /* Find the &otp= in url and insert ?nonce= before otp. Must get
-	   *  sorted headers since we calculate HMAC on the result.
-	   *
-	   * XXX this will break if the validation protocol gets a parameter that
-	   * sorts in between "nonce" and "otp", because the headers we sign won't
-	   * be alphabetically sorted if we insert the nonce between "nz" and "otp".
-	   * Also, we assume that everyone will have at least one parameter ("id=")
-	   * before "otp" so there is no need to search for "?otp=".
-	   */
-	  otp_offset = strstr (ykh->url_exp[i], "&otp=");
-	  if (otp_offset == NULL)
-	    {
-	      /* point at \0 at end of url in case there is no otp */
-	      otp_offset = ykh->url_exp[i] + len;
-	    }
-
-	  /* break up ykc->url where we want to insert nonce */
-	  *otp_offset = 0;
-
-	  wrote =
-	    snprintf (nonce_url, len, "%s" ADD_NONCE "%s&%s", ykh->url_exp[i],
-		      nonce, otp_offset + 1);
-	  if (wrote < 0 || (size_t) wrote + 1 != len)
-	    {
-	      free (nonce_url);
-
-	      out = YKCLIENT_FORMAT_ERROR;
-	      goto finish;
-	    }
-
-	  free (ykh->url_exp[i]);
-	  ykh->url_exp[i] = nonce_url;
+	  ret = ykclient_expand_new_url(ykc->url_templates[i],
+	      encoded_otp, nonce, ykc->client_id, &ykh->url_exp[i]);
 	}
-
+      if(ret != YKCLIENT_OK)
+        {
+          out = ret;
+          goto finish;
+        }
       if (ykc->key && ykc->keylen)
 	{
 	  if (!signature)
@@ -1376,7 +1460,14 @@ ykclient_verify_otp_v2 (ykclient_t * ykc_in,
 
   if (urlcount != 0 && *urls != 0)
     {
-      ykclient_set_url_templates (ykc, urlcount, urls);
+      if(strstr(urls[0], ADD_OTP "%s"))
+	{
+	  ykclient_set_url_templates (ykc, urlcount, urls);
+	}
+      else
+	{
+	  ykclient_set_url_bases (ykc, urlcount, urls);
+	}
     }
 
   if (api_key)
